﻿using System;
using System.Linq;
using System.Runtime.InteropServices.WindowsRuntime;
using System.Text;
using System.Threading.Tasks;
using Windows.Devices.Bluetooth.Rfcomm;
using Windows.Devices.Enumeration;
using Windows.Devices.HumanInterfaceDevice;
using Windows.Devices.PointOfService;
using Windows.Devices.Scanners;
using Windows.Networking.Sockets;
using Windows.Storage;
using Windows.Storage.Streams;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Media.Imaging;

// The Blank Page item template is documented at http://go.microsoft.com/fwlink/?LinkId=234238

namespace Win81Hardware
{
    /// <summary>
    /// An empty page that can be used on its own or navigated to within a Frame.
    /// </summary>
    public sealed partial class MainPage : Page
    {
		private HidDevice _legoHidDevice;
		private StreamSocket _socket;
		private ImageScanner _scanner;

		private byte[] _beepReport = { 0x0F, 0x00, 0x02, 0x00, 0x80, 0x00, 0x00, 0x94, 0x01, 0x81, 0x02, 0x82, 0xE8, 0x03, 0x82, 0x90, 0x01 };
		private byte[] _firmwareReport = { 0x0B, 0x00, 0x99, 0x00, 0x00, 0x10, 0x81, 0x0A, 0x81, 0x10, 0xE1, 0x00 };
		private byte[] _colorCommand = { 0xff, 0xfe, 0x2, 0x20, 0x1, 0x5, 0xff, 0xff, 0xff, 0x0, 0x00 };

        public MainPage()
        {
            this.InitializeComponent();
        }

		private async void MagstripeConnect_Click(object sender, RoutedEventArgs e)
		{
			try
			{
				MagneticStripeReader reader = await MagneticStripeReader.GetDefaultAsync();
				if(reader == null)
					throw new Exception("No reader found or mainfest not modified.");

				ClaimedMagneticStripeReader claimedReader = await reader.ClaimReaderAsync();
				claimedReader.BankCardDataReceived += claimedReader_BankCardDataReceived;
				claimedReader.IsDecodeDataEnabled = true;
				await claimedReader.EnableAsync();
				Output.Text = "Connected to MagStripe reader";
			}
			catch (Exception ex)
			{
				Output.Text = ex.Message;
			}
		}

		private async void claimedReader_BankCardDataReceived(ClaimedMagneticStripeReader sender, MagneticStripeReaderBankCardDataReceivedEventArgs args)
		{
			await WriteOutput2(args.FirstName + " " + args.Surname);
		}



		private async void BarcodeConnect_Click(object sender, RoutedEventArgs e)
		{
			try
			{
				BarcodeScanner scanner = await BarcodeScanner.GetDefaultAsync();
				if(scanner == null)
					throw new Exception("No scanner found or mainfest not modified.");

				ClaimedBarcodeScanner claimedScanner = await scanner.ClaimScannerAsync();
				claimedScanner.DataReceived += claimedScanner_DataReceived;
				claimedScanner.IsDecodeDataEnabled = true;
				await claimedScanner.EnableAsync();
				Output.Text = "Connected to Barcode Scanner";
			}
			catch (Exception ex)
			{
				Output.Text = ex.Message;
			}
		}

		private async void claimedScanner_DataReceived(ClaimedBarcodeScanner sender, BarcodeScannerDataReceivedEventArgs args)
		{
			DataReader dr = DataReader.FromBuffer(args.Report.ScanDataLabel);
			await WriteOutput2(dr.ReadString(args.Report.ScanDataLabel.Length));
		}



		private async void ScannerConnect_Click(object sender, RoutedEventArgs e)
		{
			try
			{
				DeviceInformationCollection devices = await DeviceInformation.FindAllAsync(DeviceClass.ImageScanner);
				DeviceInformation device = devices.FirstOrDefault();
				if(device == null)
					throw new Exception("No scanners found.");

				_scanner = await ImageScanner.FromIdAsync(device.Id);
				if(_scanner == null)
					throw new Exception("Scanner not found or mainfest not modified.");

				Output.Text = "Connected!";
			}
			catch (Exception ex)
			{
				Output.Text = ex.Message;
			}
		}

		private async void ScannerPreview_Click(object sender, RoutedEventArgs e)
		{
			IRandomAccessStream stream = new InMemoryRandomAccessStream();
			await _scanner.ScanPreviewToStreamAsync(ImageScannerScanSource.Default, stream);
			BitmapImage img = new BitmapImage();
			await img.SetSourceAsync(stream);
			Preview.Source = img;
		}

		private async void ScannerScan_Click(object sender, RoutedEventArgs e)
		{
			try
			{
				await _scanner.ScanFilesToFolderAsync(ImageScannerScanSource.Default, KnownFolders.PicturesLibrary);
			}
			catch (Exception ex)
			{
				Output.Text = ex.Message;
			}
		}

		private async void LegoConnect_Click(object sender, RoutedEventArgs e)
		{
			try
			{
           		string selector = HidDevice.GetDeviceSelector(0xff00, 0x0001, 0x0694, 0x0005);
				DeviceInformationCollection devices = await DeviceInformation.FindAllAsync(selector);
				var brick = devices.FirstOrDefault();
				if(brick == null)
					throw new Exception("No LEGO EV3 bricks found.");

				_legoHidDevice = await HidDevice.FromIdAsync(brick.Id, FileAccessMode.ReadWrite);
				if(_legoHidDevice == null)
					throw new Exception("You didn't set your manifest!");

				_legoHidDevice.InputReportReceived += _legoHidDevice_InputReportReceived;

				Output.Text = "Connected!";
			}
			catch (Exception ex)
			{
				Output.Text = ex.Message;
			}
		}

		private async void LegoBeep_Click(object sender, RoutedEventArgs e)
		{

			HidOutputReport outputReport = _legoHidDevice.CreateOutputReport();
			_beepReport.CopyTo(0, outputReport.Data, 1, _beepReport.Length);
			await _legoHidDevice.SendOutputReportAsync(outputReport);
		}

		private async void LegoFirmware_Click(object sender, RoutedEventArgs e)
		{
			HidOutputReport outputReport = _legoHidDevice.CreateOutputReport();
			_firmwareReport.CopyTo(0, outputReport.Data, 1, _firmwareReport.Length);
			await _legoHidDevice.SendOutputReportAsync(outputReport);
		}

		private async void _legoHidDevice_InputReportReceived(HidDevice sender, HidInputReportReceivedEventArgs args)
		{
            byte[] data = args.Report.Data.ToArray();

			if(data[3] == 0x99)
				await WriteOutput2(Encoding.UTF8.GetString(data, 6, 6));

			await WriteOutput2(BitConverter.ToString(data));
		}



		private async void SpheroConnect_Click(object sender, RoutedEventArgs e)
		{
			try
			{
				string selector = RfcommDeviceService.GetDeviceSelector(RfcommServiceId.SerialPort);
				DeviceInformationCollection devices = await DeviceInformation.FindAllAsync(selector);
				var device = (from d in devices where d.Name == "Sphero-WBP" select d).FirstOrDefault();
				if(device == null)
				{
					Output.Text = "Sphero not found or manifest not set.";
					return;
				}

				var service = await RfcommDeviceService.FromIdAsync(device.Id);
				_socket = new StreamSocket();
				await _socket.ConnectAsync(service.ConnectionHostName, service.ConnectionServiceName,
					 SocketProtectionLevel.BluetoothEncryptionAllowNullAuthentication);
			}
			catch (Exception ex)
			{
				Output.Text = ex.Message;
			}

			Output.Text = "Connected to Sphero";
		}

		// details from http://developer.nokia.com/Community/Wiki/Controlling_Sphero_with_Windows_Phone_8
		private async void SpheroColor_Click(object sender, RoutedEventArgs e)
		{
			for(byte b = 0; b < 0xff; b++)
			{
				_colorCommand[6] = b;
				_colorCommand[7] = (byte)~b;
				_colorCommand[8] = 0xff;

				await SendSpheroCommand(_colorCommand);
			}
		}

        private async Task SendSpheroCommand(byte[] command)
        {
            byte checksum = 0;

            for (int i = 2; i < command.GetLength(0) - 1; i++)
                checksum += command[i];

            command[command.GetLength(0) - 1] = (byte)~checksum;

            await _socket.OutputStream.WriteAsync(command.AsBuffer());
            await _socket.OutputStream.FlushAsync();
        }

		private async Task WriteOutput(string s)
		{
			await Dispatcher.RunAsync(Windows.UI.Core.CoreDispatcherPriority.Normal, () => Output.Text = s );
		}

		private async Task WriteOutput2(string s)
		{
			await Dispatcher.RunAsync(Windows.UI.Core.CoreDispatcherPriority.Normal, () => Output2.Text = s );
		}
    }
}
